﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace CurrentInput_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		CurrentInput currIn = null;
		private ErrorEventBox errorBox;
		formCleaner cleaner;

		private DataGraphBox currentGraphBox;

		public Form1() {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);

			currentGraphBox = new DataGraphBox(this, "Current");

			currIn = new CurrentInput();

			currIn.Attach += currIn_attach;
			currIn.Detach += currIn_detach;
			currIn.Error += currIn_error;
			currIn.CurrentChange += currIn_change;
			currIn.PropertyChange += currIn_propertychange;

			commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

			try { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
				currIn.Channel = phidgetParameters.Channel; //selects the channel on the device to open
				currIn.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
				currIn.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
				currIn.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

				if (phidgetParameters.isRemote) //are we trying to open a remote device?
				{
					currIn.IsRemote = true;
					Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
					if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
						Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
				} else
					currIn.IsLocal = true;

				currIn.Open(); //open the device specified by the above parameters
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			currIn.Close();
		}


		void currIn_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			settingsBox.Visible = true;
			outputBox.Visible = true;
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			currentGraphBox.SetTitle(DataGraphBox.GenerateTitle("Current", (Phidget)currIn));

			CurrentInput attachedDevice = (CurrentInput)sender;
			switch (attachedDevice.DeviceID) {
			case DeviceID.PN_DAQ1400:
				Enum[] supportedPowerSupplies;
				PowerSupplyPanel.Visible = true;
				supportedPowerSupplies = new Enum[] {PowerSupply.Off,
															 PowerSupply.Volts_12,
															 PowerSupply.Volts_24};
				powerSupplyCmb.DataSource = supportedPowerSupplies.ToList();
				powerSupplyCmb.SelectedValue = PowerSupply.Off;
				break;
			}

			try {
				//configure form controls
				dataIntervalTrk.Unit = " ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				//differing max/min means the interval is a settable configuration
				if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval)
					dataIntervalTrk.Enabled = false;

				changeTriggerTrk.Unit = "A";
				changeTriggerTrk.minTxt = attachedDevice.MinCurrentChangeTrigger.ToString();
				changeTriggerTrk.maxTxt = attachedDevice.MaxCurrentChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinCurrentChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxCurrentChangeTrigger;
				changeTriggerTrk.Value = attachedDevice.CurrentChangeTrigger;
			} catch (PhidgetException ex) { errorBox.addMessage("Error initializing device: " + ex.Message); }

			updateGraphDataInterval();
		}

		void currIn_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();
			PowerSupplyPanel.Visible = false;
			settingsBox.Visible = false;
			outputBox.Visible = false;
		}

		void currIn_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation || e.Code == ErrorEventCode.OutOfRange)
				currentGraphBox.addGap();
		}

		private void currIn_propertychange(object sender, PropertyChangeEventArgs e)
		{

			switch (e.PropertyName)
			{
				case "DataInterval":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
					{
						dataIntervalTrk.Value = currIn.DataInterval;
						updateGraphDataInterval();
					}
					break;
				case "CurrentChangeTrigger":
					changeTriggerTrk.Value = currIn.CurrentChangeTrigger;
					break;
				case "DataRate":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
					{
						dataIntervalTrk.Value = currIn.DataRate;
						updateGraphDataInterval();
					}
					break;
			}
		}

		void currIn_change(object sender, Phidget22.Events.CurrentInputCurrentChangeEventArgs e) {
			valueTxt.Text = e.Current.ToString("f4") + " A";

			currentGraphBox.addData(e.Current);
		}

		private void updateGraphDataInterval() {
			try {
				if (currIn.CurrentChangeTrigger == 0)
					currentGraphBox.DataInterval = currIn.DataInterval;
				else
					currentGraphBox.DataInterval = null;
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating graph data interval: " + ex.Message); }
		}

		private void dataIntervalTrk_Scroll_1(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					currIn.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

		private void dataIntervalTrk_MouseUp(object sender, MouseEventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					currIn.DataInterval = (int)Math.Round(dataIntervalTrk.Value);
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					currIn.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data rate: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}


		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					currIn.CurrentChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggertrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				currIn.CurrentChangeTrigger = changeTriggerTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void powerSupplyCmb_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				currIn.PowerSupply = (PowerSupply)Enum.Parse(typeof(PowerSupply), ((ComboBox)sender).SelectedValue.ToString());
			} catch (Exception ex) {
				errorBox.addMessage("Error setting voltage range: " + ex.Message);
			}
		}

		private void currentGraphButton_Click(object sender, EventArgs e) {
			if (!currentGraphBox.Visible)
				currentGraphBox.Show();
		}

		private void dataIntervalTrk_SetClick(object sender, EventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					currIn.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					currIn.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}

		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						currIn.DataInterval = (int)dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						currIn.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			}
		}

		private void changeTriggerTrk_SetClick(object sender, EventArgs e) {
			try {
				currIn.CurrentChangeTrigger = changeTriggerTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void changeTriggerTrk_SetEnter(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					currIn.CurrentChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

        private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e)
        {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.isFixedRange = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.Maximum = currIn.MaxDataRate;
					dataIntervalTrk.Minimum = currIn.MinDataRate;
					dataIntervalTrk.maxTxt = currIn.MaxDataRate.ToString();
					dataIntervalTrk.minTxt = currIn.MinDataRate.ToString("F4");
					dataIntervalTrk.trkText = currIn.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = currIn.DataRate;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data rate: " + ex.Message); }

			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					dataIntervalTrk.isIntValue = true;
					dataIntervalTrk.isFixedRange = true;
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.Maximum = currIn.MaxDataInterval;
					dataIntervalTrk.Minimum = currIn.MinDataInterval;
					dataIntervalTrk.maxTxt = currIn.MaxDataInterval.ToString();
					dataIntervalTrk.minTxt = currIn.MinDataInterval.ToString();
					dataIntervalTrk.trkText = currIn.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = currIn.DataInterval;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
    }
}